// file      : tests/cxx/parser/recursive/driver.cxx
// author    : Boris Kolpackov <boris@codesynthesis.com>
// copyright : Copyright (c) 2006-2007 Code Synthesis Tools CC
// license   : GNU GPL v2 + exceptions; see accompanying LICENSE file

// Test recursive parser invocation.
//

#include <iostream>
#include <string>

#include "test.hxx"

using std::cerr;
using std::cout;
using std::endl;
using std::string;


struct sub_parser: sub_type<void, void, void, void, string>
{
  virtual void
  pre ()
  {
    cout << "sub::pre" << endl;
  }

  virtual void
  sub ()
  {
    cout << "sub::sub" << endl;
  }

  virtual void
  sub2 ()
  {
    cout << "sub::sub2" << endl;
  }

  virtual void
  name (string const& n)
  {
    cout << "sub::name: " << n << endl;
  }

  virtual void
  post ()
  {
    cout << "sub::post" << endl;
  }
};

struct indir_parser: indir_type<void, void, string>
{
  virtual void
  pre ()
  {
    cout << "indir::pre" << endl;
  }

  virtual void
  sub ()
  {
    cout << "indir::sub" << endl;
  }

  virtual void
  name (string const& n)
  {
    cout << "indir::name: " << n << endl;
  }

  virtual void
  post ()
  {
    cout << "indir::post" << endl;
  }
};

struct test_parser: test_type<void, void, string>
{
  virtual void
  pre ()
  {
    cout << "test::pre" << endl;
  }

  virtual void
  sub ()
  {
    cout << "test::sub" << endl;
  }

  virtual void
  name (string const& n)
  {
    cout << "test::name: " << n << endl;
  }

  virtual void
  post ()
  {
    cout << "test::post" << endl;
  }
};


int
main (int argc, char* argv[])
{
  if (argc != 2)
  {
    cerr << "usage: " << argv[0] << " test.xml" << endl;
    return 1;
  }

  try
  {
    xml_schema::string string_p;

    sub_parser sub_p;
    indir_parser indir_p;
    test_parser test_p;

    sub_p.parsers (sub_p, indir_p, sub_p, string_p);
    indir_p.parsers (sub_p, string_p);
    test_p.parsers (sub_p, string_p);

    xml_schema::document<void> doc_p (test_p, "test");

    doc_p.parse (argv[1]);
  }
  catch (xml_schema::exception const& e)
  {
    cerr << e << endl;
    return 1;
  }
  catch (std::ios_base::failure const&)
  {
    cerr << "io failure" << endl;
    return 1;
  }
}
