// file      : tests/cxx/parser/validation/all/driver.cxx
// author    : Boris Kolpackov <boris@codesynthesis.com>
// copyright : Copyright (c) 2006-2007 Code Synthesis Tools CC
// license   : GNU GPL v2 + exceptions; see accompanying LICENSE file

// Test the all compositor validation.
//

#include <string>
#include <iostream>

#include "test.hxx"

using namespace std;
using namespace test;

struct all_parser: all<void,
                       string, // a
                       string, // b
                       string> // c
{
  virtual void
  pre ()
  {
    cout << "{" << endl;
  }

  virtual void
  a (string const& v)
  {
    cout << "  a = " << v << endl;
  }

  virtual void
  b (string const& v)
  {
    cout << "  b = " << v << endl;
  }

  virtual void
  c (string const& v)
  {
    cout << "  c = " << v << endl;
  }

  virtual void
  post ()
  {
    cout << "}" << endl
         << endl;
  }
};

struct type_parser: type<void, void>
{
};

int
main (int argc, char* argv[])
{
  if (argc != 2)
  {
    cerr << "usage: " << argv[0] << " test.xml" << endl;
    return 1;
  }

  try
  {
    xml_schema::string string_p;
    all_parser all_p;
    type_parser type_p;

    all_p.parsers (string_p, string_p, string_p);
    type_p.parsers (all_p);

    xml_schema::document<void> doc_p (type_p, "test", "root");

    try
    {
      doc_p.parse (argv[1], xml_schema::flags::dont_validate);
    }
    catch (xml_schema::exception const& e)
    {
      cout << "  " << e << endl
           << "}" << endl
           << endl;
    }
  }
  catch (xml_schema::exception const& e)
  {
    cerr << e << endl;
    return 1;
  }
  catch (std::ios_base::failure const&)
  {
    cerr << "io failure" << endl;
    return 1;
  }
}
