// file      : tests/cxx/parser/validation/restriction/driver.cxx
// author    : Boris Kolpackov <boris@codesynthesis.com>
// copyright : Copyright (c) 2006-2007 Code Synthesis Tools CC
// license   : GNU GPL v2 + exceptions; see accompanying LICENSE file

// Test the restriction compositor validation.
//

#include <string>
#include <iostream>

#include "test.hxx"

using namespace std;
using namespace test;

struct base_a_parser: base_a<void,
                             string, // a
                             string, // b
                             string, // c
                             string, // x
                             string, // y
                             string> // z
{
};

struct restriction_a_parser: restriction_a<void,
                                           string, // a
                                           string, // b
                                           string, // c
                                           string, // x
                                           string, // y
                                           string> // z
{
};

struct extension_b_parser: extension_b<void,
                                       string, // a
                                       string, // b
                                       string, // c
                                       string, // x
                                       string> // y
{
};

struct restriction_b_parser: restriction_b<void,
                                           string, // a
                                           string, // b
                                           string, // c
                                           string, // x
                                           string> // y

{
};

struct type_parser: type<void, void, void>
{
};

int
main (int argc, char* argv[])
{
  if (argc != 2)
  {
    cerr << "usage: " << argv[0] << " test.xml" << endl;
    return 1;
  }

  try
  {
    xml_schema::string string_p;
    base_a_parser base_a_p;
    restriction_a_parser restriction_a_p;
    extension_b_parser extension_b_p;
    restriction_b_parser restriction_b_p;
    type_parser type_b_p;
    type_parser type_r_p;

    base_a_p.parsers (string_p, string_p, string_p,
                      string_p, string_p, string_p);

    restriction_a_p.parsers (string_p, string_p, string_p,
                             string_p, string_p, string_p);

    extension_b_p.parsers (string_p, string_p, string_p,
                           string_p, string_p);

    restriction_b_p.parsers (string_p, string_p, string_p,
                             string_p, string_p);

    type_b_p.parsers (base_a_p, extension_b_p);
    type_r_p.parsers (restriction_a_p, restriction_b_p);

    xml_schema::document<void> doc_b_p (type_b_p, "test", "root");
    xml_schema::document<void> doc_r_p (type_r_p, "test", "root");

    doc_b_p.parse (argv[1], xml_schema::flags::dont_validate);

    try
    {
      doc_r_p.parse (argv[1], xml_schema::flags::dont_validate);
    }
    catch (xml_schema::exception const& e)
    {
      cout << e << endl;
    }
  }
  catch (xml_schema::exception const& e)
  {
    cerr << e << endl;
    return 1;
  }
  catch (std::ios_base::failure const&)
  {
    cerr << "io failure" << endl;
    return 1;
  }
}
