// file      : tests/cxx/parser/validation/sequence/driver.cxx
// author    : Boris Kolpackov <boris@codesynthesis.com>
// copyright : Copyright (c) 2006-2007 Code Synthesis Tools CC
// license   : GNU GPL v2 + exceptions; see accompanying LICENSE file

// Test the sequence compositor validation.
//

#include <string>
#include <iostream>

#include "test.hxx"

using namespace std;
using namespace test;

struct sequence_parser: sequence<void,
                                 string, // a
                                 string, // b
                                 string, // c
                                 string, // d
                                 string, // e
                                 string> // f
{
  virtual void
  pre ()
  {
    cout << "{" << endl;
  }

  virtual void
  a (string const& v)
  {
    cout << "  a = " << v << endl;
  }

  virtual void
  b (string const& v)
  {
    cout << "  b = " << v << endl;
  }

  virtual void
  c (string const& v)
  {
    cout << "  c = " << v << endl;
  }

  virtual void
  d (string const& v)
  {
    cout << "  d = " << v << endl;
  }

  virtual void
  e (string const& v)
  {
    cout << "  e = " << v << endl;
  }

  virtual void
  f (string const& v)
  {
    cout << "  f = " << v << endl;
  }

  virtual void
  _start_any_element (xml_schema::ro_string const& ns,
                      xml_schema::ro_string const& name)
  {
    cout << "  any: " << ns << "#" << name << endl
         << "  {" << endl;
  }

  virtual void
  _any_characters (xml_schema::ro_string const& v)
  {
    cout << "    chars = " << v << endl;
  }

  virtual void
  _end_any_element (xml_schema::ro_string const&,
                    xml_schema::ro_string const&)
  {
    cout << "  }" << endl;
  }

  virtual void
  post ()
  {
    cout << "}" << endl
         << endl;
  }
};

struct type_parser: type<void, void>
{
};

int
main (int argc, char* argv[])
{
  if (argc != 2)
  {
    cerr << "usage: " << argv[0] << " test.xml" << endl;
    return 1;
  }

  try
  {
    xml_schema::string string_p;
    sequence_parser sequence_p;
    type_parser type_p;

    sequence_p.parsers (string_p, string_p, string_p,
                        string_p, string_p, string_p);
    type_p.parsers (sequence_p);

    xml_schema::document<void> doc_p (type_p, "test", "root");

    try
    {
      doc_p.parse (argv[1], xml_schema::flags::dont_validate);
    }
    catch (xml_schema::exception const& e)
    {
      cout << "  " << e << endl
           << "}" << endl
           << endl;
    }
  }
  catch (xml_schema::exception const& e)
  {
    cerr << e << endl;
    return 1;
  }
  catch (std::ios_base::failure const&)
  {
    cerr << "io failure" << endl;
    return 1;
  }
}
