// file      : xsd/cxx/parser/characters-validation-template.cxx
// author    : Boris Kolpackov <boris@codesynthesis.com>
// copyright : Copyright (c) 2005-2007 Code Synthesis Tools CC
// license   : GNU GPL v2 + exceptions; see accompanying LICENSE file

#include <cxx/parser/characters-validation-template.hxx>

#include <xsd-frontend/semantic-graph.hxx>
#include <xsd-frontend/traversal.hxx>

namespace CXX
{
  namespace Parser
  {
    namespace
    {
      //
      //
      struct Complex : Traversal::Complex,
                       protected virtual Context
      {
        Complex (Context& c)
            : Context (c)
        {
        }

        virtual Void
        traverse (Type& c)
        {
          if (!c.mixed ())
            return;

          // @@ At the moment we are unable to generate proper code for
          // complex types with only wildcards (no templete args).
          {
            Boolean htp (false);
            HasTemplateParam t (*this, htp);
            t.traverse (c);

            if (!htp)
              return;
          }

          String const& name (ename (c));

          os <<"// Character validation functions for " << name << "." << endl
             <<"//" << endl
             << "namespace _xsd_" << name
             << "{";

            // _characters_impl
            //
            os << "template < ";

            {
              TemplateParamDeclList template_param_decl_list (*this);
              template_param_decl_list.traverse (c);
            }

            os << " >" << endl
               << "bool " << name;

            {
              TemplateParamUseList template_param_use_list (*this);
              template_param_use_list.traverse (c);
            }

            os << "::" << endl
               << "_characters_impl (const " << string_type << "& s)"
               << "{"
               << "this->_any_characters (s);"
               << "return true;"
               << "}";

            os << "}";
        }
      };
    }

    Void
    generate_characters_validation_template (Context& ctx)
    {
      Traversal::Schema schema;

      Traversal::Sources sources;
      Traversal::Names schema_names;

      Namespace ns (ctx);
      Traversal::Names names;

      schema >> sources >> schema;
      schema >> schema_names >> ns >> names;

      Complex complex (ctx);

      names >> complex;

      schema.dispatch (ctx.schema_root);
    }
  }
}

